@@ -144,6 +144,7 @@ main( int argc, char **argv )
   unsigned configlineno;
   ctrl_t ctrl;
 
+  early_system_init ();
   set_strusage (my_strusage);
   log_set_prefix ("gpgv", 1);
 
@@ -161,7 +162,6 @@ main( int argc, char **argv )
   gnupg_init_signals (0, NULL);
 
   opt.command_fd = -1; /* no command fd */
-  opt.pgp2_workarounds = 1;
   opt.keyserver_options.options |= KEYSERVER_AUTO_KEY_RETRIEVE;
   opt.trust_model = TM_ALWAYS;
   opt.batch = 1;
@@ -215,7 +215,7 @@ main( int argc, char **argv )
   ctrl = xcalloc (1, sizeof *ctrl);
 
   if ((rc = verify_signatures (ctrl, argc, argv)))
-    log_error("verify signatures failed: %s\n", g10_errstr(rc) );
+    log_error("verify signatures failed: %s\n", gpg_strerror (rc) );
 
   xfree (ctrl);
 
@@ -379,7 +379,7 @@ get_session_key (PKT_pubkey_enc *k, DEK *dek)
 {
   (void)k;
   (void)dek;
-  return G10ERR_GENERAL;
+  return GPG_ERR_GENERAL;
 }
 
 /* Stub: */
@@ -388,7 +388,7 @@ get_override_session_key (DEK *dek, const char *string)
 {
   (void)dek;
   (void)string;
-  return G10ERR_GENERAL;
+  return GPG_ERR_GENERAL;
 }
 
 /* Stub: */
@@ -399,7 +399,7 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek)
   (void)procctx;
   (void)ed;
   (void)dek;
-  return G10ERR_GENERAL;
+  return GPG_ERR_GENERAL;
 }
 
 
@@ -420,7 +420,7 @@ check_secret_key (PKT_public_key *pk, int n)
 {
   (void)pk;
   (void)n;
-  return G10ERR_GENERAL;
+  return GPG_ERR_GENERAL;
 }
 
 /* Stub:
